'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = discover;

var _bonjour = require('bonjour');

var _bonjour2 = _interopRequireDefault(_bonjour);

var _ip = require('ip');

var _ip2 = _interopRequireDefault(_ip);

var _superagent = require('superagent');

var _superagent2 = _interopRequireDefault(_superagent);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const reModel = /NC450 Admin/m;
const reServer = /lighttpd/;

/**
 * Use bonjour advertisements to find and list NC450 cameras on the network.
 *
 * @export
 * @param {number} [timeout=60000]  Stop discovery after given milliseconds has elapsed. Use -1 to run forever.
 * @param {func} callback           Function to call for every discovered camera.
 * @returns
 */
function discover(timeout = 60000, callback = undefined) {
  let discoveredHosts = [];

  // Start bonjour discovery and return a promise.
  return new Promise(resolve => {
    // Use Bonjour service discovery to find http servers on the network.
    const bonjour = new _bonjour2.default().find({ type: 'http' }, service => {
      // Check each found IPv4 address for a valid camera host
      service.addresses.filter(_ip2.default.isV4Format).forEach(address => {
        _superagent2.default.get(`http://${address}/`).end((err, response) => {
          if (!err && response.ok) {
            // Check if service response contains the fingerprint text and expected header value
            if (reModel.test(response.text) && reServer.test(response.headers.server)) {
              if (!discoveredHosts.includes(address)) {
                discoveredHosts.push(address);
              }

              // Trigger callback with the discovered service
              if (typeof callback === 'function') {
                callback(address);
              }
            }
          }
        });
      });
    });

    // Stop the discovery process after the set timeout has elapsed.
    if (timeout > -1) {
      setTimeout(() => {
        bonjour.stop();
        resolve(discoveredHosts);
      }, timeout);
    }
  });
}